/*

TAR.PRC

Bruce E. Hansen
Department of Economics
Social Science Building
University of Wisconsin
Madison, WI 53706-1393
bhansen@ssc.wisc.edu
http://www.ssc.wisc.edu/~bhansen/



This program estimates a p'th order threshold autoregression and tests the
hypothesis of a linear autoregression, using the statistics described in
"Inference when a nuisance parameter is not identified under the null
hypothesis."

The procedure is a function of eight inputs, and takes the form

  {tests,pvalues} = tar(y,p,omit,r1,r2,rep,out,graph);

The inputs are:

y     =  the vector of observations
p     =  the order of the autoregression
omit  =  lags (below p) to omit from autoregression [0 implies an AR(p)]
r1    =  the lower quantile
r2    =  the upper quantile
rep   =  number of simulation replications
out   =  indicator variable for screen output
graph =  indicator variable for graphical output


Additional detail on inputs:

p       should be a positive integer

omit    if 0, then an AR(p) model is used
        if a positive integer, then that AR lag is omitted from model
        if a vector of positive integers, then all these lags are omitted

out     should be either 0 or 1
        if 0, then nothing is printed to the screen nor to an output file
        if 1, then OLS estimates and test statistics are printed to the
            screen and to an output file (if previously declared)

graph   should be either 0 or 1
        if 0, then no graphs are generated
        if 1, then graphs of the test statistic sequences are displayed

There are two output vectors, the first contains the test statistics, and the
second the estimated asymptotic p-values.  The statistics (in order) are:
SupLM, ExpLM, AveLM, SupLMs, ExpLMs, AveLMs.

*/

proc(2) = tar(dat,p,omit,r1,r2,rep,ot,gr);

local t,k,y,x,j,s,m,mi,bnull,u,sig,xu,v,se,ses,t1,t2,tn,xr,k1,
lm,lms,yx,sim,d,xs,ys,r,br,ur,xruw,vrwt,dm,rm,olslab,
xrulm,vrlmt,wlms,sigr,vht,sims,missim,xusim,dri,vrw,
mr,xsr,xt,xti,xs1,sig1,sig2,xtu,misim,xrsim,xsu,
wrep,mrep,zt,tnk,t1r,xz1,x1,sim1,zi,zr,xz,xz2,zrt,
dtnr,supwk,indv,arv,ars,tests,pvalues,dr,su,ar,sel,kx,
fmt16,tnames,z,th,reg1,reg2,bss,rg,thn,dmn,ttp,ro,tk1,tk2,
wmax,wsum,wsume,lmmax,lmsum,lmsume,smax,ssum,ssume,srep,
supws,suplms,xmi,wh,lmh,vrwh,vrlmh,xtt1,xtt2,mrir,mririr;

@ Construct Data @
t=rows(dat)-p;
if omit==0;
 k=p;
else;
 ro=rows(omit);
 k=p-ro;
endif;
k1=k+1;
tk1=t-k1;
tk2=t-k1*2;
y=dat[1+p:t+p];
x=ones(t,1);
j=1;do while j<=p;
if sumc(omit.==j)==0;
 x=x~dat[1+p-j:t+p-j];
endif;
j=j+1;endo;

@ Null Model @
s=x'y;
m=moment(x,0);
mi=invpd(m);
bnull=mi*s;
u=y-x*bnull;
sig=(u'u)/tk1;

if ot==1;
ar=seqa(1,1,p);
if sumc(omit.>0)>0;
 sel=zeros(p,1);
 sel[omit]=ones(ro,1);
 ar=delif(ar,sel);
endif;
if p<10;
 ars=ftocv(ar,1,0);
else;
 ars=ftocv(ar,2,0);
endif;
arv=0 $+ "Y(t-" $+ ars $+ ")";
indv="C"|arv;
let olslab[1,3]= "Variable" "Estimate" "S.E.";
let fmt16[1,3]="#*.*lG" 16 8;

kx=sumc((x').*(mi*(x')));
xu=x.*(u./(1-kx));
xu=xu-(meanc(xu)');
v=moment(xu,0);
se=sqrt(diag(mi*v*mi));
output on;
"OLS Estimation of Null Linear Model";
"";
su=printfm(olslab,(0~0~0),fmt16);"";
su=printfm((indv~bnull~se),(0~1~1),fmt16);"";
"";
"Residual Variance " sig;
"";"";"";
output off;
endif;

@ Test Statistic Storage @

t1=round(t*r1);
t2=round(t*r2);
tn=t2-t1;

lm=zeros(tn,k); lms=lm;  sigr=lm;
lmmax=zeros(rep,1); lmsum=lmmax; lmsume=lmmax;
smax=lmmax; ssum=lmmax; ssume=lmmax;


sim=rndn(t,rep);
xu=x.*u;
xmi=x*mi;
misim=xmi'sim;
missim=mi*(xu'sim);

d=1;do while d<=k;            @ Pick Threshold Variable   @

if ot==1;
"Searching over Threshold Variable:" $ftocv(d,1,0);
endif;

z=rankindx(x[.,d+1],1);
zi=sortc((z~seqa(1,1,t)),1);
zr=zi[.,2];

xr=x.*(z.<=t1);
xz1=zr[1:t1];
x1=x[xz1,.];
mr=moment(x1,0);
sim1=sim[xz1,.];

xrsim=x1'sim1;
xusim=xu[xz1,.]'sim1;

r=1;do while r<=tn;           @ Pick Threshold   @

t1r=t1+r;
xz=zr[1:t1r];
xz2=zr[t1r+1:t];
zrt=zr[t1r];
xsr=x[zrt,.];
xr[zrt,1:k1]=xsr;
mr=mr+xsr'xsr;
xt=xr-xmi*mr;
mrir=mr*mi*mr;
xti=invpd(mr-mrir);
xtu=xr'u;
br=xti*xtu;
ur=u-xt*br;
sigr[r,d]=ur'ur;

xrulm=xt.*u;
vrlmt=invpd(moment(xrulm,0));
lm[r,d]=xtu'vrlmt*xtu;                 @  LM Statistic  @
lms[r,d]=(xtu'xti*xtu)/sig;                 @ Standard LM Statistic   @

@ Simulation Manipulations @

xrsim=xrsim+xsr'sim[zrt,.];
sims=xrsim-mr*misim;
wrep=sumc(sims.*(xti*sims));
smax=maxc((smax')|(wrep'));
ssum=ssum+wrep;
ssume=ssume+exp(wrep/2);

xusim=xusim+xu[zrt,.]'sim[zrt,.];
sims=xusim-mr*missim;
wrep=sumc(sims.*(vrlmt*sims));
lmmax=maxc((lmmax')|(wrep'));
lmsum=lmsum+wrep;
lmsume=lmsume+exp(wrep/2);

r=r+1; endo;
d=d+1; endo;

clear sim,misim,missim;

@ Test Statistics @
tests =
maxc(maxc(lm))|ln(meanc(meanc(exp(lm/2))))|meanc(meanc(lm))|
maxc(maxc(lms))|ln(meanc(meanc(exp(lms/2))))|meanc(meanc(lms));

@ P-values @

tnk=tn*k;
ssum=ssum/tnk;
lmsum=lmsum/tnk;
ssume=ln(ssume/tnk);
lmsume=ln(lmsume/tnk);

pvalues=meanc((lmmax~lmsume~lmsum~smax~ssume~ssum) .> (tests'));
clear lmmax,lmsume,lmsum,smax,ssume,ssum;

if ot==1;

dm=minindc(minc(sigr));
rm=minindc(sigr[.,dm]);
yx=sortc((y~x),dm+2);
ys=yx[.,1];
xs=yx[.,2:k+2];
xr=(xs[1:t1+rm,.]|zeros(t-t1-rm,k1))~(zeros(t1+rm,k1)|xs[t1+rm+1:t,.]);
dri=invpd(moment(xr,0));
br=dri*(xr'ys);
ur=ys-xr*br;
kx=sumc((xr').*(dri*(xr')));
xu=xr.*(ur./(1-kx));
vrw=moment((xu-(meanc(xu)')),0);
se=sqrt(diag(dri*vrw*dri));
sig=(ur'ur)/(t-2*k1);
sig1=(ur[1:t1+rm]'ur[1:t1+rm])/(t1+rm-k1);
sig2=(ur[t1+rm+1:t]'ur[t1+rm+1:t])/(t-t1-rm-k1);

output on;
"Global Estimates";
"Threshold Variable Lag " dm;
th=yx[t1+rm,dm+2];
"Threshold Estimate " th;
"Error Variance     " sig;
"";
bss=br~se;
dmn=ftocv(dm,(1+(dm.>9)),0);
thn=ftocv(th,6,6);
reg1="Regime 1: Y(t-" $+ dmn $+ ") < " $+ thn;
reg2="Regime 2: Y(t-" $+ dmn $+ ") > " $+ thn;
$reg1;
su=printfm(olslab,(0~0~0),fmt16);   "";
su=printfm((indv~bss[1:k1,.]),(0~1~1),fmt16); "";
"";
"Regime 1 Error Variance   " sig1;
"";
$reg2;
su=printfm(olslab,(0~0~0),fmt16);   "";
su=printfm((indv~bss[k+2:2*k1,.]),(0~1~1),fmt16);"";
"";
"Regime 2 Error Variance   " sig2;
"";"";
let tnames=
"SupLM" "ExpLM" "AveLM"
"SupLMs" "ExpLMs" "AveLMs";
ttp=tnames~tests~pvalues;
"Test Statistics and Estimated Asymptotic P-Values";
"";
"Robust LM Statistics";
su=printfm(ttp[1:3,.],(0~1~1),fmt16);"";
"Standard LM Statistics";
su=printfm(ttp[4:6,.],(0~1~1),fmt16);"";
"";
output off;
endif;

if gr==1;
_pdate="";
repeat:
d=1; do while d<=k;
xlabel(arv[d]);
z=sortc(yx[.,d+2],1);
zt=z[t1+1:t2];
ylabel("Pointwise LM Statistics");
if d==1;
title("Figure 1");
elseif d==2;
title("Figure 2");
elseif d==3;
title("Figure 3");
endif;
_plegctl={2 5 6.2 6};
_plegstr="Standard\000Hetero-Robust";
xy(zt,lms[.,d]~lm[.,d]);
d=d+1;endo;
"Do you wish to re-view the graphs (Y/N)";
rg=cons;
if vals(rg)==89 or vals(rg)==121; goto repeat; endif;
endif;


retp(tests,pvalues);
endp;
