/*
TAR_CI.PRG
To estimate and test a threshold bi-variate VECM

written by:

Bruce E. Hansen
Department of Economics
Social Science Building
University of Wisconsin
Madison, WI 53706-1393
bhansen@ssc.wisc.edu
http://www.ssc.wisc.edu/~bhansen/

and 

Byeongseon Seo
Department of Economics
Soongsil University
Seoul, 156-743
Korea
seo@saint.soongsil.ac.kr


This Gauss program estimates a bi-variate VECM, a threshold bi-variate VECM, 
and tests for the presence of a threshold.  The methods are those described in
"Testing for Threshold Cointegration" by Bruce E. Hansen and Byeongseon Seo.

The program is set up to replicate the empirical application to the 3-month and 6-month
interest rates series.  For your own application, load your data into the matrix "dat", and change the
controls listed below


*/

/*************************************************************/

@  Controls @

k=1;			@ Lags in VAR beyond EC @
gn=300;			@ number of gridpoints for gamma @
bn = 300;		@ number of gridpoints for beta      @
_trim=.05;		@ trimming percentage for threshold @
boot=5000;		@ number of bootstrap replications 
		     set equal to zero to not do testing  @
_coint=1;		@ set to 1 to estimate cointegrating vector
			  set to 0 to fix cointegrating vector at _cvalue @
_cvalue=1;		@ cointegrating vector, if _coint=0 @
_cov=1;		@ covariance matrix estimation method
		   set to 1 for Eicker-White
		   set to 0 for conventional homoskedastic estimator @
_p_ests=1;		@ set to 1 to print estimates, else 0 @
_graph=1;		@ set to 1 to generate graph of nonlinear ECM, else 0 @
_graph_rotate=0;	@ set to 1 to generate rotated graphs 
		     (useful for some print jobs, but ackward for screen viewing) @
output file=tar_ci.out; 	@ name of output file @

@ Load your own data  into matrix "dat" @
@ Here we load in the 3-month and 6-month T-Bill series @

load dat[482,62]=zeroyld.dat;
dat=dat[1:rows(dat),7:62];
rs = seqa(0,1,19)|21|24|30|seqa(36,12,8);
short=12;
long=120;
short_i=maxindc(rs.==short);
long_i=maxindc(rs.==long);
dat=dat[.,long_i short_i];    

/*************************************************************
MAIN PROGRAM
*************************************************************/
if _p_ests==1;
format 12,6;
output on;
"**************************************************";"";
"Number of Bootstrap Replications   " boot;
"Number of Gridpoints for threshold " gn;
if _coint==1;
"Estimated Cointegrating Vector";
"Number of Gridpoints for ci vector " bn;
else;
"Cointegrating Vector fixed at " _cvalue;
endif;
"";"";
"Long Rate  (month):     " long;
"Short Rate (month):      " short;
"Number of VAR lags:   " k;
"";
output off;
endif;

@ Organize Data @
n=rows(dat);
y = dat[2+k:n,.]-dat[1+k:n-1,.];
t=rows(y);
xlag=dat[1+k:n-1,.];
x=ones(t,1);
j=1; do while j<=k;
  x=x~(dat[2+k-j:n-j,.]-dat[1+k-j:n-1-j,.]);
j=j+1;endo;

@ Compute Linear Model@
xx=invpd(moment(x,0));
xxx=x*xx;
u=y-x*xx*(x'y);
if _coint==1;
  v=xlag-x*xx*(x'xlag);
  uu=moment(u,0);
  m=invpd(moment(v,0))*(v'u)*invpd(uu)*(u'v);
  {va,ve}=eigv(m);
  h=ve[.,maxindc(va)];
  b0=-h[2]/h[1];
else;
  b0=_cvalue;
endif;
w0=xlag*(1|-b0);
z0 = w0~x;
kk=cols(z0);
zz0=invpd(moment(z0,0));
zzz0=z0*zz0;
beta0=zzz0'y; 
e = y - z0*beta0; 
sige=moment(e,0)/t;
nlike=(t/2)*ln(det(sige));
bic=nlike+log(t)*4*(1+k);
aic=nlike+2*4*(1+k);
nlike1=b_like(b0+.001);
nlike2=b_like(b0-.001);
hp=(nlike1+nlike2-2*nlike)/(.001^2);
seb=1/sqrt(hp);
if _cov ==1;
  xe=(z0.*e[.,1])~(z0.*e[.,2]);
  m0=eye(2).*.zz0;
  v=m0*moment(xe,0)*m0;
else;
  v=sige.*.zz0;
endif;
se=sqrt(diag(v));

if _p_ests==1;
output on;
"Linear VECM Estimates";
"";
if _coint==1;
"Cointegrating Vector " b0~seb;
else;
"Cointegrating Vector " b0;
endif;
"Negative Log-Like " nlike;
"AIC                        " aic;
"BIC                        " bic;
"";
"  Equation 1";
beta0[.,1]~se[1:kk];
"";
"  Equation 2";
beta0[.,2]~se[kk+1:2*kk];
"";"";"";
output off;
endif;

@ Set-up Grids @
q=unique(w0,1);
gamma1=q[round(seqa(_trim,(1-2*_trim)/gn,gn)*rows(q))];
gamma2=q[round(seqa(1/(gn+1),1/(gn+1),gn)*rows(q))];
if _coint==1;
  if bn==1; betas=b0;
  else;
  betas = seqa(b0-6*seb,12*seb/(bn-1),bn);
  endif;
else;
  betas=b0;bn=1;
endif;

@ Estimate Threshold Model @

store=zeros(gn,bn)+nlike;
j=1; do while j<=gn;
  gam=gamma2[j];
  bj=1; do while bj<=bn;
    beta=betas[bj];
    w=xlag*(1|-beta);
    z = w~x;
    d1=(w .<= gam);n1=sumc(d1);
    if minc(n1|(t-n1))/t > _trim;
      zj=(z.*d1)~w;
      zzj=zj-x*xx*(x'zj);
      trap 1; bz=u/zzj; trap 0;
      if scalerr(bz); bz=pinv(moment(zzj,0))*(zzj'u); endif;
      store[j,bj]=(t/2)*ln(det(moment(u-zzj*bz,0)/t));
    endif;
  bj=bj+1;endo;
j=j+1;endo;
m=minindc(store);
c=minindc(diag(store[m,.]));
r=m[c];
gammahat=gamma2[r];
b1=betas[c];
nlike=store[r,c];
bic=nlike+log(t)*8*(1+k);
aic=nlike+2*8*(1+k);

  w=xlag*(1|-b1);
  z = w~x;
  d1=(w .<= gammahat);
  d2=1-d1;
  y1=selif(y,d1);
  z1=selif(z,d1);
  y2=selif(y,d2);
  z2=selif(z,d2);
  zz1=invpd(moment(z1,0));zz2=invpd(moment(z2,0));
  beta1=zz1*(z1'y1); beta2=zz2*(z2'y2);
  e1 = y1 - z1*beta1;
  e2 = y2 - z2*beta2;
  if _cov==1;
    xe1=(z1.*e1[.,1])~(z1.*e1[.,2]);
    m1=eye(2).*.zz1;
    v1=m1*moment(xe1,0)*m1;
    xe2=(z2.*e2[.,1])~(z2.*e2[.,2]);
    m2=eye(2).*.zz2;
    v2=m2*moment(xe2,0)*m2;
  else;
    sig1=moment(e1,0)/rows(e1);
    v1=sig1.*.zz1;
    sig2=moment(e2,0)/rows(e2);
    v2=sig2.*.zz2;
  endif;
  se1=sqrt(diag(v1));
  se2=sqrt(diag(v2)); 
  rb=rows(beta1);
  bb=beta1-beta2;
  if k>0;
    bw=vec(bb[3:rb,.]);
    vr=seqa(3,1,rb-2)|seqa(rb+3,1,rb-2);
    vv=v1[vr,vr]+v2[vr,vr];
    ww=bw'invpd(vv)*bw;
  endif;
  wecm=bb[1,.]*invpd(v1[1 rb+1,1 rb+1]+v2[1 rb+1,1 rb+1])*(bb[1,.]');

if _p_ests==1;
output on;
"Threshold VECM Estimates";
"";
"Threshold Estimate " gammahat;
"Cointegrating Vector Estimate " b1;
"Negative Log-Like " nlike;
"AIC                        " aic;
"BIC                        " bic;
"";
"First Regime";
"Percentage of Obs" meanc(d1);
"";
"  Equation 1";
beta1[.,1]~se1[1:kk];
"";
"  Equation 2";
beta1[.,2]~se1[kk+1:2*kk];
"";
"Second Regime";
"Percentage of Obs" meanc(d2);
"";
"  Equation 1";
beta2[.,1]~se2[1:kk];
"";
"  Equation 2";
beta2[.,2]~se2[kk+1:2*kk];
"";
if k>0;
"Wald Test for Equality of Dynamic Coefs" ww~cdfchic(ww,(rb-2)*2);
endif;
"Wald Test for Equality of ECM Coef " wecm~cdfchic(wecm,2);"";"";
output off;
endif;

/***************************************************************/

if (_graph==1) or (_graph_rotate==1);

graphset;
pqgwin many;
_pdate="";
_plwidth=2;
_pcolor=15;
fonts("complex simgrma");

if _coint==1;
  title("Figure 2\LConcentrated Negative Log Likelihood");
  ylabel("Negative Log-Likelihood");
  loglike=minc(store);
  xlabel("Beta");
  xy(betas,loglike);
  if _graph_rotate==1;
    _protate=1; 
    xy(betas,loglike);
    _protate=0; 
  endif;
endif;

title("Figure 1\LConcentrated Negative Log Likelihood");
ylabel("Negative Log-Likelihood");
xlabel("Gamma");
loglike=minc(store');
xy(gamma2,loglike);
if _graph_rotate==1;
  _protate=1; 
  xy(gamma2,loglike);
  _protate=0; 
endif;

yhat2=(z[.,1 2]*beta1[1 2,.]).*d1+(z[.,1 2]*beta2[1 2,.]).*d2;
wi=sortind(w);
yhat2=yhat2[wi,.];
wi=w[wi];
ylong = "" $+ "R]" $+ ftocv(long,1,0) $+ "[";
yshort = "" $+ "R]" $+ ftocv(short,1,0) $+ "[";
ylong_t = ylong $+"(t-1)";
yshort_t = yshort $+ "(t-1)";
if _coint==0;
  xtit= "" $+ "Error Correction: " $+ ylong_t $+ "-"$+ yshort_t;
elseif _coint==1;
  xtit= "" $+ "Error Correction: " $+ ylong_t $+ "-\202b\201*"$+ yshort_t;
endif;
xlabel(xtit);
ylabel("Interest Rate Response");
title("Figure 3\LInterest Rate Response to Error-Correction");
_plegctl=1;
_plegstr="" $+ "Long Rate (" $+ ylong $+ ")\000Short Rate (" $+ yshort $+ ")";
xy(wi,yhat2);

if _graph_rotate==1;
  _protate=1; 
  xy(wi,yhat2);
  _protate=0; 
endif;

endif;

/********************************************************/

@ Testing @

lm01=lmtest01(y,x,w0,gamma1,_graph);
if _p_ests==1;
output on;
"Lagrange Multipler Threshold Test";
"";
"Test Statistic                                                       " lm01;
output off;
endif;

if boot>0;
@ Fixed Regressor Bootstrap @
fix01=zeros(boot,1);
r=1; do while r<=boot;
  yr=rndn(t,2).*e;
  fix01[r]=lmtest01(yr,x,w0,gamma1,0);
r=r+1;endo;
pfix01=meanc(fix01.>lm01)';
fix01=sortc(fix01,1);
cfix01=fix01[round(.95*boot),.];

@ Parametric Bootstrap  @
x0=dat[1:1+k,.];
mu=beta0[2,.];
ab=(1|(-b0))*beta0[1,.];
boot01=zeros(boot,1);
if k>0;
  capgamma=beta0[3:2+2*k,.];
  dx0=vecr(rev(x0[2:k+1,.]-x0[1:k,.]));
endif;
r=1; do while r<=boot;
  datb=x0;
  if k>0;   dx=dx0; endif;
  datbi=datb[k+1,.];
  ei=ceil(rndu(t,1)*t);
  eb=e[ei,.];
  i=k+2; do while i<=n;
    u=mu+eb[i-k-1,.]+datbi*ab;
    if k>0; u=u+dx'capgamma; endif;
    datbi=datbi+u;
    datb=datb|datbi;
    if k==1;
      dx=u';
    elseif k>1;
      dx=(u')|dx[1:2*k-2];
    endif;
  i=i+1;endo;
  yr = datb[2+k:n,.]-datb[1+k:n-1,.];
  xlagr=datb[1+k:n-1,.];
  xr=ones(t,1);
  j=1; do while j<=k;
    xr=xr~(datb[2+k-j:n-j,.]-datb[1+k-j:n-1-j,.]);
  j=j+1;endo;
  if _coint==1;
    xxr=invpd(moment(xr,0));
    u=yr-xr*xxr*(xr'yr);
    v=xlagr-xr*xxr*(xr'xlagr);
    m=invpd(moment(v,0))*(v'u)*invpd(moment(u,0))*(u'v);
     {va,ve}=eigv(m);
    h=ve[.,maxindc(va)];
    wr=xlagr*(1|(h[2]/h[1]));
  else;
    wr=xlagr*(1|-_cvalue);
  endif;
  boot01[r]=lmtest01(yr,xr,wr,gamma1,0);
r=r+1;endo;
boot01=sortc(boot01,1);
cb01=boot01[round(.95*boot)];
pb01=meanc(boot01 .> lm01)';

if _p_ests==1;
output on;
"Fixed Regressor (Asymptotic) 5% Critical Value " cfix01;
"Bootstrap 5% Critical Value                                " cb01;
"Fixed Regressor (Asymptotic) P-Value                " pfix01;
"Bootstrap P-Value                                               " pb01;
"";"";
output off;
endif;

endif;

/***************************************************************/

proc b_like(b);
local z,sigma,like;
  z = (xlag*(1|-b))~x;  
  sigma=moment(y - z*(y/z),0)/t;
  like=(t/2)*ln(det(sigma));
retp(like);endp;


@ Proc for LM test @
proc lmtest01(y,x,w0,gammas,_plot);
local s,e,ze,store,j,d1,n1,z1,z11,v,lm01,sv,z0zz,e1,e2,z0;
z0=w0~x;
trap 1; z0zz=((z0')/moment(z0,0))'; trap 0;
if scalerr(z0zz); z0zz=z0*pinv(moment(z0,0)); endif;
e=y-z0zz*(z0'y); 
e1=e[.,1];e2=e[.,2];
store=zeros(gn,1);
j=1; do while j<=gn;
    d1=(w0 .<= gammas[j]);
    n1=sumc(d1);
    if minc(n1|(t-n1))/t > _trim;
      z1=z0.*d1;
      z11=z1-z0zz*(z0'z1);
      ze=(z11.*e1)~(z11.*e2);
      v=moment(ze,0);
      s=vec(z11'y);
      trap 1; sv=s/v; trap 0;
      if scalerr(sv); sv=pinv(v)*s; endif;
      store[j]=s'sv;
    endif;
j=j+1;endo;
lm01=maxc(store)';
if _plot==1;
  graphset;
  _pdate="";
  _plwidth=2;
  _pcolor=15;
  fonts("complex simgrma");
 xlabel("Gamma");
 title("LM Statistic as function of Gamma");
  xy(gammas,store);
  if _graph_rotate==1;
    _protate=1; 
    xy(gammas,store);
    _protate=0; 
  endif;
endif;
retp(lm01);
endp;
